package test;

import java.util.Collections;

import org.jbpm.api.ProcessInstance;
import org.jbpm.api.ProcessInstanceQuery;
import org.jbpm.api.task.Task;
import org.jbpm.test.JbpmTestCase;


public class ComplateTaskTest extends JbpmTestCase {

  String deploymentId;
  
  protected void setUp() throws Exception {
 
    super.setUp();
  
    deploymentId = repositoryService.createDeployment()
    .addResourceFromClasspath("test/CreateClient.jpdl.xml")
    .deploy();
  }

  protected void tearDown() throws Exception {
    repositoryService.deleteDeploymentCascade(deploymentId);
    super.tearDown();
  }
  
  public void testCreateClientApprove() {
    ProcessInstance pi = executionService.startProcessInstanceByKey("CreateClient");
    
    //signal first state activity
    pi = executionService.signalExecutionById(pi.getId(), Collections.singletonMap("client", "testClient"));
    // signal second state activity
    pi = executionService.signalExecutionById(pi.getId());
    //complete task
    Task task = taskService.createTaskQuery().uniqueResult();
    
    taskService.completeTask(task.getId(), "decision", Collections.singletonMap("decision", "approve"));
    
    
  }
  
  public void testCreateClientReject() {
    ProcessInstance pi = executionService.startProcessInstanceByKey("CreateClient");
    
    //signal first state activity
    pi = executionService.signalExecutionById(pi.getId(), Collections.singletonMap("client", "testClient"));
    // signal second state activity
    pi = executionService.signalExecutionById(pi.getId());
    //complete task
    Task task = taskService.createTaskQuery().uniqueResult();
    
    taskService.completeTask(task.getId(), "decision", Collections.singletonMap("decision", "reject"));
    
    
  }
  
  public void testCreateClientRejectSignalWithBusinessKey() {
    ProcessInstance pi = executionService.startProcessInstanceByKey("CreateClient", Collections.singletonMap("client", "testClient"), "CreateClient-1");
    
    //signal first state activity
    signalExecutionByBusinessKey("CreateClient-1", "ClientAdded");
    // signal second state activity
    signalExecutionByBusinessKey("CreateClient-1", "ClientUpdated");
    //complete task
    Task task = taskService.createTaskQuery().uniqueResult();
    
    taskService.completeTask(task.getId(), "decision", Collections.singletonMap("decision", "reject"));
    
  }
  
  private void signalExecutionByBusinessKey(String key, String activityName){
    // defaulted the state with error. no, m not being pessimistic :-)
    String newState = "Error - failed to signal the instance";
    
    // query the process instance
    ProcessInstanceQuery query = executionService.createProcessInstanceQuery();
    ProcessInstance pi = query.processInstanceKey(key).list().get(0);
    
    
    if(pi != null){
    // signal PI to move forward
        String id = pi.getId();
        
        if (id == null) {
             id = pi.findActiveExecutionIn(activityName).getId();
        }
        pi = executionService.signalExecutionById(pi.getId());
        //newState = pi.geta
        
    }
    
}

}
